#!env python
"""
Factory to produce xmllib objects
"""

__author__ = "Nick Knouf <nknouf@desnak.com>"
__version__ = "$Revision: 1.9 $"
__date__ = "$Date: 2004/10/12 00:55:47 $"
__copyright__ = "Copyright 2004, desnak"
##__license__ = "Python"

import cStringIO
import sys

class xmllib(object):
    """Interface to common xml functions."""

    def __init__(self):
        pass

    def parse(self):
        raise NotImplementedError

    def getChildren(self):
        pass

    def getRoot(self):
        pass
   
    def getName(self, node):
        raise NotImplementedError

    def get_node_values(self, nodes):
        raise NotImplementedError

    def xpath(self, doc, xpath):
        raise NotImplementedError

    def free(self):
        raise NotImplementedError

    def freeNode(self, node):
        pass

    def getTextReader(self):
        pass

    def setRelaxNGSchema(self):
        pass

    def getRelaxNGParser(self):
        pass

    def setContent(self):
        raise NotImplementedError

    def newNode(self, namespace=None, name=None):
        pass

    def serialize(self, document):
        raise NotImplementedError

    def serializeToFile(self, document, filename):
        pass

    def copyNode(self):
        pass

    def addChild(self, parent=None, child=None):
        pass

    def addChildNodeList(self, parent=None, child=None):
        pass

    def newChild(self, parent=None, namespace=None, name=None, content=None):
        raise NotImplementedError

    def getContent(self):
        raise NotImplementedError

    def validate(self, schema, file, type='relaxng'):
        pass

    def _readFile(self, filename):
        """Helper function to read in a file.

Sometimes needed to read in a schema file, xml file, etc.
        """
        try:
            fd = open(filename, 'r')
        except IOError:
            return None

        file = fd.read()
        fd.close()

        return file

class libxml2Product(xmllib):
    """Concrete libxml2 interface for our xmllib."""
    
    def __init__(self):
        import libxml2
        import libxml2mod

        super(libxml2Product, self).__init__()
    
        self.libxml2 = libxml2
        self.libxml2mod = libxml2mod

    def newDoc(self):
        """Create a new document object."""

        return self.libxml2.newDoc('1.0')

    def setRoot(self, document, root):
        """Set the root of a document object."""

        return document.setRootElement(root)

    def newNode(self, namespace=None, name=None):
        """Create a new xml node."""

        return self.libxml2.newNode(name)

    def copyNode(self, node):
        """Copy the current node."""

        return node.copyNode(False)

    def parse(self, thing, type='filename'):
        """Parse the given xml file."""
        try:
            if type == 'filename':
                return self.libxml2.parseFile(thing)
            elif type == 'string':
                return self.libxml2.parseMemory(thing, len(thing))
        except self.libxml2.parserError:
            return None
    
    def getTextReader(self, filename):
        """Returns a text reader given a filename."""
        try:
            return self.libxml2.newTextReaderFilename(filename)
        except self.libxml2.treeError:
            return None

    def setRelaxNGSchema(self, reader, schema):
        """Set the reader to a specific relax NG schema."""

        try:
            reader.RelaxNGSetSchema(schema)
        except AttributeError:
            return None

    def getRelaxNGParser(self, filename):
        """Get a relax NG parser object from the file 'filename'."""

        schema = self._readFile(filename)

        if schema is not None:
            parser = self.libxml2.relaxNGNewMemParserCtxt(schema, len(schema))
            parser = parser.relaxNGParse()
            return parser
        else:
            return None

    def setContent(self, node, content):
        """Set the content of a particular node."""

        node.setContent(content)

    def newProp(self, node, name, value):
        """Create a new property(attribute)."""

        node.newProp(name, value)

    def addChild(self, parent=None, child=None):
        """Add the node `child` as a child of the node `parent`."""

        if (parent or child) is None:
            raise NodeNodeError, "must give a parent and a child node."

        return parent.addChild(child)

    def addChildNodeList(self, parent=None, child=None):
        """Add the node set `child` as a child of the node `parent`."""

        if (parent or child) is None:
            raise NodeNodeError, "must give a parent and a child node."

        return parent.addChildList(child)

    def newChild(self, parent=None, namespace=None, name=None, content=None):
        """Create a new child node."""

        if parent is None:
            raise NoNodeError, "must give a parent node to add a child node."

        if name is None:
            raise NoNameError, "must give a name for the child node."

        if namespace is not None:
            parent.newTextChild(namespace, name, content)
        else:
            parent.newTextChild(None, name, content)

    def getContent(self, node):
        """Get the content of a particular node."""

        return node.getContent()

    def xpath(self, doc, xpath):
        """Evaluate an xPath expression."""
        return doc.xpathEval(xpath)

    def getRoot(self, doc):
        """Get the root element; return as a node object."""

        return doc.getRootElement()

    def serialize(self, document):
        """Serialize the document; defaults to indenting."""

        return document.serialize(format=True)

    def serializeToFile(self, document, filename):
        """Serialize `document` to file `filename`; returns the serialized string."""

        string = document.serialize(format=True)
        document.saveFile(filename)
        return string

    def validate(self, schema, document, type='relaxng', schema_format='file', document_format='file'):
        """Validate `document` using schema `schema`.

        Defaults to assuming `schema` is in relaxNG format."""

        # turn off error reporting for invalid documents
        def noerr(ctx, str):
            pass
        self.libxml2.registerErrorHandler(noerr, None)

        if schema_format.lower() == "string":
            schema_fd = cStringIO.StringIO(schema)
        elif schema_format.lower() == "file":
            schema_fd = open(schema, 'r')
        
        if document_format.lower() == "string":
            document_fd = cStringIO.StringIO(document)
        elif document_format.lower() == "file":
            document_fd = open(document, 'r')

        # read in the schema
        schema_file = schema_fd.read()
        schema_fd.close()

        if type.lower() == "relaxng":
            result = self._validate_relaxng(schema_file, document_fd)

            # potential cleanup
            if document_format.lower() == "file": document_fd.close()

            return result

    def _validate_relaxng(self, schema, document):
        """Private function to do the relaxNG validation."""

        # fun libxml2 stuff with contexts...
        # parse the schema
        rngp = self.libxml2.relaxNGNewMemParserCtxt(schema, len(schema))
        rngs = rngp.relaxNGParse()

        # read and parse the document
        input_read = self.libxml2.inputBuffer(document)
        reader = input_read.newTextReader("urn:ddd")
        reader.RelaxNGSetSchema(rngs)

        ret = reader.Read()
        while ret == 1:
            ret = reader.Read()

        if ret != 0:
            #raise XmllibParseError, "Error parsing document"
            raise "Error parsing document"

        if reader.IsValid() == 1:
            return True
        else:
            return False

    def free(self, doc):
        """Free the desired document."""

        doc.freeDoc()

    def freeNode(self, node):
        """Free the desired node."""

        node.freeNode()


class xmllibFactory(object):
	"""Factory to create a specific xmllib object."""

	def create(factoryName):
		"""Create the desired xmllib from a given name.
		
		Found the method to do this from the excellent book, available online, DiveIntoPython (http://diveintopython.org)"""

		module = sys.modules[xmllibFactory.__module__]
		subclass = "%sProduct" % factoryName
		return (hasattr(module, subclass) and getattr(module, subclass))

	create = staticmethod(create)
