# Copyright (C) 2004 nick knouf <nknouf@mit.edu>
#  
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software 
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: glade.py,v 1.2 2004/11/10 02:02:38 commit Exp $
#

__author__ = "Nick Knouf <nknouf@desnak.com>"
__version__ = "$Revision: 1.2 $"
__date__ = "$Date: 2004/11/10 02:02:38 $"
__copyright__ = "Copyright 2004, Nick Knouf"
__license__ = "GPL2"

# make sure all of our imports work correctly
try:
    import pygtk
    #tell pyGTK, if possible, that we want GTKv2
    pygtk.require("2.0")
except:
    #Some distributions come with GTK2, but not pyGTK
    pass

try:
    import gtk
    import gtk.glade
except:
    print "You need to install pyGTK or GTKv2 ",
    print "or set your PYTHONPATH correctly."
    print "try: export PYTHONPATH=",
    print "/usr/local/lib/python2.2/site-packages/"
    sys.exit(1)

try:
    import gobject                                    
except:                                              
    print "don't have gobject?"                      
    sys.exit(1)

# now we have both gtk and gtk.glade imported
# Also, we know we are running GTK v2

# standard library import
import time

# get the location of the glade file
from __init__ import ddd_glade_location

class gui(object):
    """GUI class using glade and GTK."""

    def __init__(self, entries):
        """Setup the interface and establish callbacks"""

        self.entries = entries

        global ddd_glade_location
        gladefile = ddd_glade_location
        windowname = "mainWindow"
        self.wTree = gtk.glade.XML(gladefile, windowname)

        self.initialize_listview(self.wTree)
        self.populate_listview(self.entries)

        signals_dict = {"on_menu_edit_delete_entry_activate": \
                        self.menu_edit_delete_entry_handler, \
                        "on_menu_file_quit_activate": \
                        (self.menu_file_quit_handler, self.entries), \
                        "on_menu_help_about_activate": \
                        self.menu_help_about_handler, \
                        "on_mainWindow_destroy": \
                        (self.menu_file_quit_handler, self.entries)}

        self.statusbar_left = self.wTree.get_widget('statusbar_left')
        self.statusbar_left.push(self.statusbar_left.get_context_id('statusbar_left'), "Successfully started.")

        self.wTree.signal_autoconnect(signals_dict)

        return

    def initialize_listview(self, tree):
        """Setup the columns for the list view."""

        # create the model
        self.listview = tree.get_widget("entryList")
        self.listmodel = gtk.ListStore(gobject.TYPE_PYOBJECT,
                                        gobject.TYPE_STRING,
                                        gobject.TYPE_STRING,
                                        gobject.TYPE_STRING)
        self.listview.set_model(self.listmodel)

        # now setup necessary properties
        self.listview.set_headers_visible(gtk.TRUE)
        self.listview.set_headers_clickable(gtk.TRUE)

        renderer=gtk.CellRendererText()
        column=gtk.TreeViewColumn("Entry Type",renderer, text=1)
        column.set_resizable(gtk.TRUE)
        self.listview.append_column(column)

        renderer=gtk.CellRendererText()
        column=gtk.TreeViewColumn("Date",renderer, text=2)
        column.set_resizable(gtk.TRUE)
        column.connect("clicked", self.perform_sort)
        self.listview.append_column(column)

        renderer=gtk.CellRendererText()              
        column=gtk.TreeViewColumn("Title",renderer, text=3)
        column.set_resizable(gtk.TRUE)               
        self.listview.append_column(column)          
                                                     
        # setup "clicked" callback                   
        self.listview.connect("button_press_event", self.on_entryList_clicked)

        # setup sorting callback
        #self.listmodel.connect("sort-column-changed", self.perform_sort)
        selection = self.listview.get_selection()
        selection.connect("changed", self.update_entryText)

        # display everything                         
        self.listview.show()

    def populate_listview(self, entries):
        """Populate the listview with the entries stored in `entries`."""

        for entry in entries.project_nodes:
            entry.get_attributes()
            self.insert_row(self.listmodel, \
                            None, \
                            entry, \
                            "Project", \
                            time.ctime(float(entry.modified)), \
                            entry.get_entryTitle())

        for entry in entries.design_nodes:
            entry.get_attributes()
            self.insert_row(self.listmodel, \
                            None, \
                            entry, \
                            "Design", \
                            time.ctime(float(entry.modified)), \
                            entry.get_entryTitle())

        for entry in entries.code_nodes:
            entry.get_attributes()
            self.insert_row(self.listmodel, \
                            None, \
                            entry, \
                            "Code", \
                            time.ctime(float(entry.modified)), \
                            entry.get_entryTitle())

        for entry in entries.thoughts_nodes:
            entry.get_attributes()
            self.insert_row(self.listmodel, \
                            None, \
                            entry, \
                            "Thoughts", \
                            time.ctime(float(entry.modified)), \
                            entry.get_entryTitle())

        # sort by date
        self.listmodel.set_sort_column_id(2, gtk.SORT_ASCENDING)

        self.listview.show()
        
    def perform_sort(self, column):
        """Perform the sort function."""
        print "got signal"
        listmodel.set_sort_column_id(column, gtk.SORT_ASCENDING)

    def on_entryList_clicked(self, widget, event):
        """Callback for "clicked" event in entryList."""
        if event.type == gtk.gdk._2BUTTON_PRESS:
            (model, row) = widget.get_selection().get_selected()
            #entry = self.listmodel.get_value(row, 0)
            #print "type: %s; date: %s; title: %s" % (type, date, title)
            
            self.edit_entry(row)

    def _get_widget_tree_by_window(self, windowName):
        """Return the widget tree for the given windowName."""
        global ddd_glade_location
        return gtk.glade.XML(ddd_glade_location, windowName)


    def edit_entry(self, row):
        """Edit an entry."""
        global ddd_glade_location
        gladefile = ddd_glade_location
        windowname = "entryWindow"
        self.editTree = gtk.glade.XML(gladefile, windowname)

        signals_dict = {"on_entryCancel_clicked": \
                        self.entryCancel_clicked_handler, \
                        "on_entrySave_clicked": \
                        self.entrySave_clicked_handler, \
                        "on_entryTitle_editing_done": \
                        self.print_entryTitle}
        #"on_entryTitle_changed": \
        #self.print_entryTitle,  \

        self.editTree.signal_autoconnect(signals_dict)
        
        # get the data from the model
        self.current_entry = self.listmodel.get_value(row, 0)
        self.current_type = self.listmodel.get_value(row, 1)
        self.current_date = self.listmodel.get_value(row, 2)
        self.current_title = self.listmodel.get_value(row, 3)

        # update the window
        editWindow = self.editTree.get_widget("entryWindow")
        editWindow.set_title("edit '%s'" % self.current_title)

        entryTitle = self.editTree.get_widget('entryTitle')
        entryTitle.set_text(self.current_title)

        entryKeywords = self.editTree.get_widget('entryKeywords')
        self.current_entryKeywords = ", ".join(self.current_entry.get_keywords())
        entryKeywords.set_text(self.current_entryKeywords)

        entryText = self.editTree.get_widget('entryText')
        buffer = entryText.get_buffer()
        self.current_entryText = '\n\n'.join(self.current_entry.get_content())
        buffer.set_text(self.current_entryText)
        entryText.set_buffer(buffer)

    def entryCancel_clicked_handler(self, widget):
        """When the user clicks 'cancel', make sure there are not unsaved changes; if there are, prompt the user to save."""
        
        entryTitle = self.editTree.get_widget("entryTitle").get_text()
        entryKeywords = self.editTree.get_widget("entryKeywords").get_text()

        # hoops to jump through to get our text
        textBuffer = self.editTree.get_widget("entryText").get_buffer()
        numCharacters = textBuffer.get_char_count()
        start = textBuffer.get_iter_at_offset(0)
        end = textBuffer.get_iter_at_offset(numCharacters)
        entryText = textBuffer.get_text(start, end)
        editWindow = self.editTree.get_widget("entryWindow")

        if not ((entryTitle == self.current_title) and (entryKeywords == self.current_entryKeywords) and (entryText == self.current_entryText)):
            self.display_saveWarningWindow()
            editWindow.destroy()
        else:
            editWindow.destroy()


    def entrySave_clicked_handler(self, widget):
        """Save the entry data."""

        pass

    def display_saveWarningWindow(self):
        """Display a window warning that the entry is not saved."""

        saveWarningWindowTree = self._get_widget_tree_by_window("saveWarningWindow")
        saveWarningWindow = saveWarningWindowTree.get_widget("saveWarningWindow")
        saveWarningWindow.show()
        response = saveWarningWindow.run()
        if response == gtk.RESPONSE_NO:
            saveWarningWindow.destroy()
        else:
            self.save_entry()
            saveWarningWindow.destroy()

    def save_entry(self, widget):
        """Save the entry."""

        pass

    def update_entryText(self, widget):
        """Update the entryText textview when a row is selected."""
        
        # get the content from the entry object
        (model, row) = widget.get_selected()
        if row is not None:
            entry = self.listmodel.get_value(row, 0)
            content = entry.get_content()
    
            # update the textview
            self.textview = self.wTree.get_widget("entryText")
            buffer = self.textview.get_buffer()
            buffer.set_text('\n\n'.join(content))
            self.textview.set_buffer(buffer)


    def insert_row(self, model, parent, ddd_object, col1, col2, col3):
        """Insert a row in the list."""

        myiter = model.append(None)
        model.set_value(myiter, 0, ddd_object)
        model.set_value(myiter, 1, col1)
        model.set_value(myiter, 2, col2)
        model.set_value(myiter, 3, col3)
        return myiter

    def print_entryTitle(self, widget):              
        """Print what we wrote to standard out."""   
        result = self.editTree.get_widget("entryTitle").get_text()
        print result                                 

    def editEntry_changed(self, widget):
        """Callback for when something in the editEntry window changes."""
        print "editEntry changed callback"
        # ensure the window title indicates something has changed
        self._mark_window_title(widget)

    def _mark_window_title(self, widget):
        """Ensure the window title indicates that something has changed."""
        print "should be marking the window title"
        window = widget.window
        window_title = window.get_title()

        # if the first character is a *, we don't need to do anything
        if window_title[0] != "*":
            window_title = "* " + window_title
            window.set_title(window_title)

    def menu_edit_delete_entry_handler(self, widget):
        """Delete an entry from the project file."""

        # get the selected row
        editList = self.wTree.get_widget("entryList")
        (model, row) = editList.get_selection().get_selected()

        if row is not None:
            # the user selected an entry; confirm that they want to delete it
            entry = self.listmodel.get_value(row, 0)
            entryTitle = self.listmodel.get_value(row, 3)
    
            deleteEntryWindowTree = self._get_widget_tree_by_window("deleteEntryWindow")
            deleteEntryWindow = deleteEntryWindowTree.get_widget("deleteEntryWindow")
            deleteEntryWindow.show()
            response = deleteEntryWindow.run()
            if response == gtk.RESPONSE_NO:
                # oops, don't want to delete
                deleteEntryWindow.destroy()
            else:
                # okay, remove the entry from the xml tree
                # @@TODO@@ update this to show some visual cue that the project
                # file now needs saving
                del entry
                self.listmodel.remove(row)
                self.listview.set_cursor(0)
                self.statusbar_left.push(self.statusbar_left.get_context_id('statusbar_left'), "Entry deleted.")

                deleteEntryWindow.destroy()
        else:
            # no entry selected!
            self.statusbar_left.push(self.statusbar_left.get_context_id('statusbar_left'), "No entries selected.")

    def menu_file_quit_handler(self, widget, entries):        
        """Eventually will check to make sure that file is saved, etc."""
        entries.free()

        gtk.mainquit()

        return

    def menu_help_about_handler(self, widget):
        """Display the about box."""

        aboutDialogTree = self._get_widget_tree_by_window("aboutDialog")
        aboutDialogWindow = aboutDialogTree.get_widget("aboutDialog")

        aboutDialogWindow.show()
        response = aboutDialogWindow.run()
        if response == gtk.RESPONSE_OK:
            aboutDialogWindow.destroy()

def run(gui):
    """Start the event loop."""
    app = gui
    gtk.mainloop()
