#!env python
"""
run_tests.py

Runs the complete test suite, or, if called with a series of command-line parameters, runs the specified test suites.

All tests are grouped by module, prepended with "test_".
"""

import sys

#we need to add the necessary directories to the search path, and then reverse,
# just in case the modules we're looking for already exist in the path
sys.path.append('../')
sys.path.append('../ddd/factories')
sys.path.reverse()

import unittest, os

modules = ['xmllibFactory', 'rdflibFactory', 'ValidatorFactory']

def main(args):
	"""run the full test suite, or only a subset"""

	if (len(sys.argv) > 1):
		names = []

		for module in sys.argv[1:]:
			names.append('test_%s' % module)
	else:
		files = os.listdir(os.curdir)
		names = []
		
		for file in files:
			if file[:4] == "test" and file[-3:] == ".py":
				names.append(file[:-3])

	suite = unittest.defaultTestLoader.loadTestsFromNames(names)
	runner = unittest.TextTestRunner(verbosity = 2)
	result = runner.run(suite)
	sys.exit(not result.wasSuccessful())

if __name__ == "__main__":
	main(sys.argv)
