#!env python
"""
Code to test the SitePreferences code
"""

__author__ = "Nick Knouf <nknouf@desnak.com>"
__version__ = "$Revision: 1.1 $"
__date__ = "$Date: 2004/08/23 22:07:22 $"
__copyright__ = "Copyright 2004, desnak"
##__license__ = "Python"

import unittest
import ValidatorFactory

class ValidatorTest(unittest.TestCase):
	"""Test the Validator code"""

	methods = ['validate', 'isValid']

	def setUp(self):
		self.validator = ValidatorFactory.Validator()

	def testHasRequiredFunctions(self):
		"""Does the base class have the functions that we want?"""

		for method in self.methods:
			self.assert_(hasattr(self.validator, method), 'Base Validator object does not have method "%s"' % method)


class ValidatorFactoryTest(unittest.TestCase):
	"""Test the ValidatorFactory code."""

	def setUp(self):
		pass

	def testCreatedRelaxNGObject(self):
		"""Did we create a relax NG validator object?"""

		self.validator = ValidatorFactory.ValidatorFactory().create('relaxng')

		validator = self.validator()

		self.assertEquals(validator.__class__, ValidatorFactory.relaxngValidator, "Unable to create a relax NG validator object.")

class relaxngValidatorTest(unittest.TestCase):
	"""Test the relax ng validator code."""

	def setUp(self):
		self.validator = ValidatorFactory.ValidatorFactory().create('relaxng')
		self.validator = self.validator(schema='', document='')

	def testReaderAndParserAreNone(self):
		"""Are our reader and parser None?"""

		self.assertEquals(self.validator.rng_parser, None, 'Parser is not "None".')
		self.assertEquals(self.validator.reader, None, 'Reader is not "None".')

	def testValidatorGives0(self):
		"""Does our 'validator' function return 0 with empty documents?"""

		self.valid = self.validator.validate()

		self.assert_(not self.valid, "'validate' function does not return 0 for empty documents.")


if __name__ == "__main__":
	unittest.main()
