#!env python
"""
Code to test the SitePreferences code
"""

__author__ = "Nick Knouf <nknouf@desnak.com>"
__version__ = "$Revision: 1.1 $"
__date__ = "$Date: 2004/08/23 22:07:22 $"
__copyright__ = "Copyright 2004, desnak"
##__license__ = "Python"

import unittest
import rdflibFactory
import RDF

class rdflibTest(unittest.TestCase):
	"""Test the rdflib code"""

	methods = ['createModel', 'createStorage', 'addStatement', 'createParser', 'createStatement', 'createNode', 'getStatements', 'createNamespace']

	def setUp(self):
		self.validator = rdflibFactory.rdflib()

	def testHasRequiredFunctions(self):
		"""Does the base class have the functions that we want?"""

		for method in self.methods:
			self.assert_(hasattr(self.validator, method), 'Base rdflib object does not have method "%s"' % method)


class rdflibFactoryTest(unittest.TestCase):
	"""Test the rdflibFactory code."""

	def setUp(self):
		pass

	def testCreatedRedlandObject(self):
		"""Did we create a Redland RDF library object?"""

		self.rdflib = rdflibFactory.rdflibFactory().create('Redland')

		rdflib = self.rdflib()

		self.assertEquals(rdflib.__class__, rdflibFactory.RedlandLibrary, "Unable to create a Redland RDF library object.")

class RedlandLibraryTest(unittest.TestCase):
	"""Test the Redland library code."""

	def setUp(self):
		self.rdflib = rdflibFactory.rdflibFactory().create('Redland')
		self.rdflib = self.rdflib()

	def testCreateParser(self):
		"""Do we create a default parser (commonly raptor)?"""

		parser = self.rdflib.createParser()
		
		self.assertEquals(parser.__class__, RDF.Parser, "Creating the parser did not create the correct object.")
		
	def testCreateNamespace(self):
		"""Can we create a namespace object?"""

		fooNS = self.rdflib.createNamespace("http://foo.org/bar#")

		self.assertEquals(fooNS.__class__, RDF.NS, "Creating the namespace did not create the correct object.")
	
	def testCreateNode(self):
		"""Can we create a node object?"""

		fooNode = self.rdflib.createNode("I'm a literal node.")

		self.assertEquals(fooNode.__class__, RDF.Node, "Creating the node did not create the correct object.")
		self.assert_(fooNode.is_literal(), "The node should be a literal node, but isn't.")


if __name__ == "__main__":
	unittest.main()
